@@ -17,6 +17,8 @@ extern int  _GMP_miller_rabin_random(mpz_t n, UV numbases, char* seedstr);
 
 extern void _GMP_lucas_seq(mpz_t U, mpz_t V, mpz_t n, IV P, IV Q, mpz_t k,
                            mpz_t Qk, mpz_t t);
+extern int lucas_lehmer(UV p);
+extern int llr(mpz_t N);
 
 extern UV   _GMP_trial_factor(mpz_t n, UV from_n, UV to_n);
 
@@ -40,6 +42,7 @@ extern void _GMP_primorial(mpz_t prim, mpz_t n);
 extern void _GMP_lcm_of_consecutive_integers(UV B, mpz_t m);
 extern void bernfrac(mpz_t num, mpz_t den, mpz_t n);
 extern void stirling(mpz_t r, unsigned long n, unsigned long m, UV type);
+extern void binomial(mpz_t r, UV n, UV k);
 
 extern UV   power_factor(mpz_t n, mpz_t f);
 extern UV   is_power(mpz_t n, UV a);
